
/*

	upCommentBox v1.0
	by: Unknown Person
	03/23/08

	--------------------------------------------------------------------

	upCommentBox is a plug-and-play utility and demo of the usage of
	the library upForm.  The code is includable with only a few lines
	of code allowing the client procedures to be called.  This environ-
	ment is also a demo in that it demonstrates how the functions and
	features of upForm can be used to create interesting and useful
	browser interfaces using the library.  upCommentBox also saves the
	comments into a savefile.

	Release Info:
	v1.0 (Mar 24 08)
	 - Demo released
	v1.1 (Mar 26 08)
	 - the authorized list is now updated via upCB_Authorized(), which
	   should only be called in world/New() to add keys to the authorized
	   list.  Refer to to the usage instructions. (Gughunter)

	--------------------------------------------------------------------

	Usage:

	In order to use this utility, you should allow players to use two
	main procs, and a proc to handle authorization.

	client/upCB_Submit() [proc]
	 - Call this when you want a player to submit a new comment
	client/upCB_Manage() [proc]
	 - Call this when you want an administrator to be able to manage
	   existing comments
	upCB_Authorize(key) [proc]
	 - Appends a new key to the upCB_authorized list to determine what keys
	   are authorized to manage comments.  Values are stored in canonical
	   form (therefore, the key "Unknown Person" in canonical form would
	   be "unknownperson").  This value is essentially obtained by
	   using client.ckey, or using ckey() to the key string.  The key sent
	   through the arg is converted to canonical form

	The easiest way to use library is to just define verbs for the submit
	and manage button, but only allow authorized keys to use the manage
	verb.  Don't forget to call upCB_Authorize, or nobody will be able
	to manage comments.

	Example:
	~~
	world/New()
		upCB_Authorize("Unknown Person")	// proc converts key to canonical form
		upCB_Authorize("gughunter")			// and also accepts canonical form keys
		..()
	~~

	When accessing the authorized list, use the variable definition,
	upCB_authorized.

	Example:
	~~
	if(src.ckey in upCB_authorized)
	  world << "[src] is an authorized upCommentBox manager!"
	~~

	See example.dm for an example.

	--------------------------------------------------------------------
*/

#include <UnknownPerson/upForm>
#define UPCB_SAVEFILE			"upcb_comments.sav"
#define UPCB_TIME_FORMAT		"MMM DD, 'YY"
#define UPCB_TITLELEN			30
#define UPCB_COMMENTLEN			1000

var/list
	// an associative list containing the formal name and internal name of the
	// categories a comment can be in
	upCB_categories = list("Comment"="comment",
						   "Suggestion"="suggestion",
						   "Bug Report"="bug",
						   "Note"="note")

	upComments = list() 		// global list of all comments
	upCB_authorized = list()	// list of keys authorized to view/manage comments

world
	New()
		upCB_LoadComments()
		..()

	Del()
		upCB_SaveComments()
		..()

client/proc
	upCB_Submit()
		var/upComment/C = new /upComment (src)
		upForm(src, C, /upForm/upCommentBox/submit)

	upCB_Manage()
		upForm(src, /upForm/upCommentBox/manage)

	upCB_Read(upComment/C)
		upForm(src, C, /upForm/upCommentBox/read)

proc
	upCB_validCategory(cat)
		for(var/item in upCB_categories)
			var/code = upCB_categories[item]
			if(cat == code)
				return item
		return 0

	upCB_canReadComment(upComment/C, client/A)
		return (A.ckey in upCB_authorized) || C.owner == A.key

	upCB_Authorize(key)
		key = ckey(key)
		if(key in upCB_authorized)
			return
		upCB_authorized += key

	upCB_SaveComments()
		var/savefile/F = new (UPCB_SAVEFILE)
		F["data"] << upComments

	upCB_LoadComments()
		if(!fexists(UPCB_SAVEFILE))
			return 0

		var/savefile/F = new (UPCB_SAVEFILE)
		F["data"] >> upComments
		if(!upComments)
			upComments = list()

	upCB_AddComment(upComment/C)
		if(!C) return
		C.Initialize()

	upCB_DeleteComment(upComment/C, client/A)	// called when user A attempts to delete comment C
		// only allow authorized clients to delete a comment
		if(!(A.ckey in upCB_authorized))
			return 0

		del(C)
		// update all manage forms to ensure no operations are made on a
		// deleted comment
		upForm_UpdateForms(/upForm/upCommentBox/manage)
		upCB_SaveComments()

		return 1


upComment
	var
		title = "No Title"		// given title of comment by client
		owner = ""				// key of client who submitted comment
		category = "none"		// type of comment
		severity = 0			// if bug report, severity rating (from 1 to 10)
		body = ""				// body of comment
		submit_date = 0			// submission date recorded from world.realtime.  num used for sorting
		submit_date_text		// translated text version of submit date

	New(client/_owner)
		..()

		// initialize when created using Submit(), and not when
		// comments are loaded
		if(_owner != null)
			// allow both mobs or clients to be sent through New()
			if(ismob(_owner))
				var/mob/M = _owner
				_owner = M.client
			// set owner string to the client's key
			owner = _owner.key

	Del()
		// when comments are deleted, remove them from the
		// global comment list
		upComments -= src
		..()

	proc
		Initialize()	// called when the comment has been succesfully submitted and is archived
			upComments += src

			// set date based on defined time format
			src.submit_date = world.realtime
			src.submit_date_text = time2text(world.realtime, UPCB_TIME_FORMAT)

		// returns the formal category name from the internal name
		// also append the severity rating if it's a bug
		getCategory()
			var/cat = upCB_validCategory(src.category)
			if(src.category == "bug")
				return "[cat] ([src.severity])"
			return cat


upForm/upCommentBox
	// use the same CSS for all upCommentBox forms
	page_css = {"
body, table, div {
  font-family: 'Verdana';
  font-size: 10pt;
  color: #ffffff;
}

body {
  background-color: #303030;
}

table {
  border-collapse: collapse;
}

a:link, a:visited {
  text-decoration: none;
  color: #bbddbb
}
a:hover {
  text-decoration: none;
  color: #ffffff
}

table.page {
  width: 100%;
  height: 100%;
}

tr.crow {
  height: 24px;
}

tr.categories {
  background-color: #202020;
  height: 24px;
}

table.comments {
  width: 100%;
}

div.comments {
  width: 100%;
  height: 100%;
  background-color: #2b2b2b;
  overflow: auto;
}

span.title {
  font-weight: bold;
}

span.error {
  color: #ff0000;
  font-size: 8pt;
}

input, select, textarea {
  font-family: 'Verdana';
  font-size: 10pt;
}

input.title, select.category, select.severity {
  width: 100%;
}

textarea.body {
  width: 100%;
  height: 100%;
}

input.button {
  width: 100px;
  background-color: #393939;
  border-width: 2px;
  border-color: #333333;
  color: #ffffff;
}

img.cmd {
  width: 24px;
  height: 24px;
}
	"}

	// manage form allows authorized clients to manage all the submitted comments
	manage
		window_title = "View Submitted Comments"
		window_size = "600x400"
		window_params = UPFORM_CANNOT_RESIZE
		form_type = UPFORM_WINDOW
		resources = list('upCB_delete.png')

		Link(list/href_list)
			var/action = href_list["action"]
			var/value = href_list["value"]
			switch(action)
				if("close")
					del(src)
				if("delete")
					var/upComment/C = locate(value)
					if(!C || !istype(C) || !upCB_DeleteComment(C, src.getOwner()))
						RefreshPage()
				if("read")
					var/client/A = src.getOwner()
					var/upComment/C = locate(value)
					if(C && istype(C) && upCB_canReadComment(C, A))
						A.upCB_Read(C)
						del(src)


		GenerateBody()
			var/comments = ""

			// Add the category titles first
			comments += {"
			  <tr class="crow" valign="center">
			    <td width="55%"> <b>Title / Owner</b> </td>
			    <td width="20%"> <b>Category</b> </td>
			    <td width="20%"> <b>Date</b> </td>
			    <td width="5%"> </td>
			  </tr>
			"}

			// Then generate the comment data
			var/counter = 1
			for(var/upComment/C in upComments)
				// Create a horizontal line between each row by adding a top border to every
				// row except the first one
				// We put this style in the <td> tag due to the fact IE cannot create borders
				// using <tr>
				var/line = counter++ != 1 ? {" style="border-top: 1px solid #2b2b2b;""} : ""
				comments += {"
				  <tr class="categories" valign="center">
				    <td[line]>
				      <a href="byond://?src=\ref[src]&action=read&value=\ref[C]">
				        [C.title]
				      </a> <br />
				      <i>[C.owner]</i>
				    </td>
				    <td[line]> [C.getCategory()] </td>
				    <td[line]> [C.submit_date_text] </td>
				    <td[line] align="right">
				      <a href="byond://?src=\ref[src]&action=delete&value=\ref[C]">
				        <img class="cmd" src="upCB_delete.png" alt="Delete [C.owner]'s Comment" />
				      </a>
				    </td>
				  </tr>
				"}

			/*	A div is created to allow the comment table to stretch indefinitly
				and letting us scroll using an internal scrollbar.  This allows us to
				see all the page content, while scrolling through the potential dozens
				of comments without needing to scroll back up to the top to close the
				window.  Take a look at the CSS defined in /upForm/upCommentBox to see
				what properties were defined for the div.comments class
			*/

			var/body = {"
				<table class="page">
			    <tr class="crow">
				  <td> <span class="title"> Manage Comments </span> </td>
				  <td align="right">
				    <input class="button" type="button" onclick="upF_action('close');"
				     value="Close" />
				  </td>
				</tr>
				<tr class="crow"> <td colspan="2">
				  <hr />
				</td> </tr>
				<tr class="comments"> <td colspan="2">
				  <div class="comments">
				    <table class="comments">
				      [comments]
				    </table>
				  </div>
				</td></tr>
				</table>
			"}

			UpdatePage(body)

	// read forms display all the info about a comment
	// simple interface that just displays information
	read
		window_title = "Read Comment"
		window_size = "500x325"
		window_params = UPFORM_CANNOT_RESIZE
		form_type = UPFORM_WINDOW

		PreSettings()
			// Set the window title a dynamic value before the window is
			// displayed
			var/upComment/C = src.getHost()
			src.window_title = "[C.title] ([C.owner])"

		Link(list/href_list)
			if(href_list["action"] == "close")
				del(src)

		GenerateBody()
			var/upComment/C = src.getHost()

			// Do not display the severity rating content if the category isn't a bug report
			var/body = {"
				[C.category != "bug" ? \
				  {"<body onload="document.getElementById('sev_label').style.display='none';" />"} : ""]

			  	<table class="page">
			    <tr class="crow">
				  <td colspan="2"> <span class="title"> [C.owner]'s Comment </span> </td>
				  <td colspan="2" align="right">
				    <input class="button" type="button" onclick="upF_action('close');"
				     value="Close" />
				  </td>
				</tr>
				<tr class="crow"> <td colspan="4">
				  <hr />
				</td> </tr>
				<tr class="crow">
				  <td width="20%"> <b>Title:</b> </td>
				  <td width="80%" colspan="3">
				  	[C.title]
				  </td>
				</tr>
				<tr class="crow">
				  <td width="20%"> <b>Author:</b> </td>
				  <td width="40%"> [C.owner] </td>
				  <td width="15%"> <b>Date:</b> </td>
				  <td width="25%"> [C.submit_date_text] </td>
				</tr>
				<tr class="crow">
				  <td width="20%"> <b>Category:</b> </td>
				  <td width="40%">
				  	[upCB_validCategory(C.category)]
				  </td>
				  <td width="40%" colspan="2">
				    <span id="sev_label"> <b>Severity:</b> [C.severity] </span>
				  </td>
				</tr>
				<tr class="crow">
				  <td colspan="4">
				    <b>Comment:</b>
				  </td>
				</tr>
				<tr valign="top"><td colspan="4">
				  <div class="comments" style="border: 1px solid black; padding: 10px">
				    [C.body]
				  </div>
				</td></tr>
				</table>
			"}

			UpdatePage(body)

	// submit form that allows players to submit new comments
	submit
		window_title = "Submit Comment"
		window_size = "500x325"
		window_params = UPFORM_CANNOT_RESIZE
		form_type = UPFORM_WINDOW
		settings = UPFORM_SET_HANDLE_FORMS

		Link(list/href_list)
			var/upComment/C = src.getHost()
			if(href_list["action"] == "close")
				// when closed, delete the unused comment datum, then close the form
				del(C)
				del(src)

		canDisplayForm(client/C)
			. = ..()
			if(C.upForm_isViewingForm(src.type))
				. = 0

		FormInitTempVars()
			var/upComment/C = src.getHost()
			initFormVar("submit", "title", 		C.title)
			initFormVar("submit", "category", 	C.category)
			initFormVar("submit", "severity", 	C.severity)
			initFormVar("submit", "body", 		C.body)

		FormSetTempVars(fname)
			var/upComment/C = src.getHost()
			if(fname == "submit")
				C.title 	= getFormVar("submit", "title")
				C.category 	= getFormVar("submit", "category")
				C.severity 	= getFormVar("submit", "severity")
				C.body	 	= getFormVar("submit", "body")

		FormSubmitSuccess(fname)
			// When the comment is succesfully submitted, archive it,
			// then close the form
			var/upComment/C = src.getHost()
			if(C) upCB_AddComment(C)
			del(src)

		ProcessVariable(fname, name, value)
			if(fname == "submit")
				switch(name)
					if("title")
						setFormVar(fname, name, value)
						if(length(value) < 3 || length(value) > UPCB_TITLELEN)
							return "Title must be between 3 and [UPCB_TITLELEN] characters long"
					if("category")
						setFormVar(fname, name, value)
						if(!upCB_validCategory(value))
							return TRUE
					if("severity")
						if(getFormVar("submit", "category") == "bug")
							var/num = text2num(value)
							if(num > 10 || num < 1)
								num = 0
							setFormVar(fname, name, num)
							if(num == 0)
								return "Please enter a severity rating"
						else
							setFormVar(fname, name, 0)
					if("body")
						setFormVar(fname, name, value)
						if(length(value) < 3 || length(value) > UPCB_COMMENTLEN)
							return "Body text must be between 3 and [UPCB_COMMENTLEN] characters"
			return null



		GenerateBody(list/errors=list())
			var/category_selection = {"<option value="none"> --- </option>"}
			var/severity_selection = {"<option value="0"> --- </option>"}

			for(var/item in upCB_categories)
				var/cat = getFormVar("submit", "category")
				var/code = upCB_categories[item]
				category_selection += {"<option[cat == code ? " selected" : ""]
					value="[code]"> [item] </option>"}

			for(var/c = 1 to 10)
				var/sev = getFormVar("submit", "severity")
				severity_selection += {"<option[sev == c ? " selected" : ""]
					value="[c]"> [c] </option>"}

			/* 	The following JavaScript code enables us to let the "Severity" rating
				appear only when the category is set to Bug Reports.  Since we want the
				page to change dynamically without reloading the page (due to the fact
				we are using a form, althought it is still possible), we need to use
				JavaScript.  In order to make the severity rating "invisible" while
				the category isn't a bug report, we give both the drop-down selection
				menu and the label an id.  We also define JavaScript functions that make
				the tagged items visible or invisible.  dispSeverity() directly changes
				the visibility of the objects.  It is called when the page is loaded only
				if the category is Bug Reports.  However, we use catSelect() to dynamically
				make the severity visible or invisible when a category is selected.
			*/
			var/js = {"
				function dispSeverity(s) {
				  var sevLabel = document.getElementById("sev_label");
				  var sevSelect = document.getElementById("sev_select");

				  if(sevLabel != null && sevSelect != null) {
				    sevLabel.style.display = s;
				    sevSelect.style.display = s;
				  }
				}

				function catSelect(select) {
				  var s = select.value;
				  if(s == "bug")
				    dispSeverity('block');
				  else
				    dispSeverity('none');
				}
			"}

			// Notice how all variables are refered to by using getFormVar(), because all of
			// these variables are temporary until the form is successfully submitted
			var/body = {"
				[getFormVar("submit","category") != "bug" ? \
				  {"<body onload="dispSeverity('none');" />"} : ""]

				<form name="submit" action="byond://" method="get">
				  <input type="hidden" name="src" value="\ref[src]" />
				  <input type="hidden" name="form" value="submit" />

			  	  <table class="page">
			      <tr class="crow">
				    <td colspan="2"> <span class="title"> Submit Comment </span> </td>
				    <td colspan="2" align="right">
				      <input class="button" type="button" onclick="upF_action('close');"
				       value="Close" /> <br />
				    </td>
				  </tr>
				  <tr class="crow"> <td colspan="4">
				    <hr />
				  </td> </tr>
				  <tr class="crow">
				    <td width="15%"> Title: </td>
				    <td width="85%" colspan="3">
				      <input class="title" type="text" name="title" maxlength="[UPCB_TITLELEN]"
				       value="[getFormVar("submit","title")]" />
				       <span class="error"> [errors["title"]] </span>
				    </td>
				  </tr>
				  <tr class="crow">
				    <td width="15%"> Category: </td>
				    <td width="45%">
				      <select name="category" class="category" onclick="catSelect(this)">
				        [category_selection]
				      </select>
				      <span class="error"> [errors["category"]] </span>
				    </td>
				    <td width="20%"> <span id="sev_label"> Severity: </span> </td>
				    <td width="20%">
				      <select id="sev_select" name="severity" class="severity">
				        [severity_selection]
				      </select>
				      <span class="error"> [errors["severity"]] </span>
				    </td>
				  </tr>
				  <tr class="crow">
				    <td colspan="4">
				      Comment: <span class="error"> [errors["body"]] </span>
				    </td>
				  </tr>
				  <tr>
				    <td colspan="4">
				      <textarea class="body" name="body"
				       maxlength="[UPCB_COMMENTLEN]">[getFormVar("submit","body")]</textarea>
				    </td>
				  </tr>
				  <tr class="crow">
				    <td colspan="2"> </td>
				    <td colspan="2" align="right">
				      <input class="button" type="submit" value="Submit" />
				    </td>
				  </tr>
				  </table>
				</form>
			"}

			UpdatePage(body, js)